#ifndef BST_H_
#define BST_H_

#include <stddef.h>
#include <stdbool.h>

struct bst_node
   {
   struct bst_node * parent;
   struct bst_node * left;
   struct bst_node * right;
   void * data;
   };

struct bst
   {
   struct bst_node * root;
   size_t size;
   bool (*le)(void * a, void * b);
   bool (*eq)(void * a, void * b);
   };

struct bst_iterator
   {
   struct bst * bst;
   struct bst_node * curr;
   };

struct bst * bst_new(bool (*le)(void * a, void * b), bool (*eq)(void * a, void * b));
void bst_delete(struct bst * bst);

void * bst_insert(struct bst * bst, void * data);

void * bst_search(struct bst * bst, void * data);
void * bst_search_iterative(struct bst * bst, void * data);

void * bst_minimum(struct bst * bst);
void * bst_maximum(struct bst * bst);

void bst_walk_inorder(struct bst * bst, void (*op)(void * data));
void bst_walk_preorder(struct bst * bst, void (*op)(void * data));
void bst_walk_postorder(struct bst * bst, void (*op)(void * data));


struct bst_iterator * bst_begin(struct bst * bst);
struct bst_iterator * bst_end(struct bst * bst);
struct bst_iterator * bst_next(struct bst_iterator * it);
struct bst_iterator * bst_prev(struct bst_iterator * it);
void * bst_value(struct bst_iterator * it);

#endif
